#!/usr/bin/python
# -*- coding:iso-8859-1  -*-
#
# Code utilise pour l'article http://ptaff.ca/routes/
#
# Copyright (C) 2006 Miguel Tremblay
#
#  Questions or bugs report: miguel.tremblay@ptaff.ca
#
#
# Code contributed by:
#  Miguel Tremblay - ptaff.ca
#
########################################################################
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
#


"""
 Name:	      graph_annee_degel_pluie_interval.py
 Description: Prend les fichiers de sortie de nombre_de_degel.py
  et fait un graphique avec les annees sur l'abscisse (tout les fichiers
  trouvee dans le repertoire correspondant aux seuils) et le
  nombre de degel avec pluie en ordonnee.
  La moyenne est aussi tracee pour chaque annee.

       Input: station_directory seuil1 seuil2
       Output: graph.png
 
 Notes: 

 Author: Miguel Tremblay
 Date: 13 aout 2006
"""

import sys
from rpy import r
import os.path

sDirectory2Seuils = 'degel_2_seuils/'

lYear = range(1996,2007)


dCouleur = {'bagotville':'blue',
            'ottawa':'red',
            'fredericton':'orange',
            'halifax':'cyan',
            'montreal':'green',
            'quebec':'purple',
            'toronto':'magenta', 
            'KALB':'gold',
            'KBTV':'brown'}


def main(sDirectory, sPathGraph, bSvg, nSeuilMin, nSeuilMax, bPluie):
    nYMax = 0

    # Changer la definition en cas de recherche avec precip
    if bPluie:
        sDegel = 'D,1'
    else:
        sDegel = 'D,'

    # Ouvrir le repertoire
    lDirectory =  os.listdir(sDirectory)

    dStation = {}
    for sStationDirectory in lDirectory:
        sStationPath = os.path.join(sDirectory, sStationDirectory, \
                                    sDirectory2Seuils)
        if not os.path.isdir(sStationPath):
            continue
        # Prendre seulement les fichiers qui
        #  contiennent une annee en 2e position
        dYearNbrDegel = {}
        lStationDirectory =  os.listdir(sStationPath)
        for sFilename in lStationDirectory:
            lFilename = sFilename.split('_')
            try:
                if len(lFilename) < 9 or int(lFilename[1]) not in lYear \
                       or "unique" in sFilename or \
                       int(lFilename[6].split('.')[0]) != nSeuilMin or \
                       int(lFilename[8].split('.')[0]) != nSeuilMax:
                    continue
                else:
                    file_input = open(os.path.join(sStationPath, sFilename), \
                                      'r')
                    lReadline = file_input.readlines()
                    nbr_lines = 0
                    for sLine in lReadline:
                        if sDegel in sLine:
                            nbr_lines = nbr_lines + 1
                    sYear = lFilename[1]                
                    dYearNbrDegel[sYear] = nbr_lines
                    if nbr_lines > nYMax:
                        nYMax = nbr_lines
            except ValueError:
                continue
            dStation[sStationDirectory] = dYearNbrDegel

    if len(dStation.keys()) == 0:
        print "Directory", sDirectory, "contains no data"
        sys.exit(1)

    nYMax = nYMax + int(nYMax/2.5)
    draw_graph(sPathGraph, dStation, nYMax, bSvg, nSeuilMin, nSeuilMax, bPluie)


def draw_graph(sPathGraph, dStation, nYMax, bSvg, nSeuilMin, nSeuilMax, \
               bPluie):
    """
    Dessine une graphique avec les annees en abscisse et le nombre de
    gel en ordonnee. Une ligne pour chaque station
    """
    # Label
    if bSvg:        
        sXLabel = "Ann&#233;e"
        sYLabel = "Nombre de d&#233;gels"
        if bPluie:
            sMain = "Nombre de d&#233;gels avec pr&#233;cipitation\npar" + \
                    "ann&#233;e"
            sYLabel = sYLabel + " avec pr#233;cipitation"
        else:
            sMain = "Nombre de d&#233;gels par ann&#233;e"
    else:
        sXLabel = "Anne"
        sYLabel = "Nombre de dgels"
        if bPluie:
            sMain = "Nombre de dgels avec prcipitation\npar anne"
            sYLabel = sYLabel + " avec prcipitation"
        else:
            sMain = "Nombre de dgels par anne"

    if nSeuilMax < 1000 and nSeuilMin == 0:
        sMain = sMain + ' pour un \ninterval de seuils  ' + str(nSeuilMin) +\
                '-' + str(nSeuilMax)
    elif nSeuilMax > 1000 and nSeuilMax !=0:
        sMain = sMain + ' pour un \ninterval de seuils de plus de ' +\
                str(nSeuilMin)
    else:
        sMain = sMain 

    lAxisRange = [0, nYMax]

    lStation = dStation.keys()
    dDummy = dStation[lStation[0]] 
    lKeys = dDummy.keys()
    lKeys.sort()
    lAxeX = []
    lAxeY = []
    for sKey in lKeys:
        lAxeX.append(int(sKey))
        lAxeY.append(dDummy[sKey])

    if bSvg:
        r.library("RSvgDevice")
        r.devSVG(sPathGraph, width = 8, height = 8,
          bg = "white", onefile=r.TRUE, xmlHeader=r.TRUE )
    else:
        r.png(sPathGraph, width = 480, height = 480,
              pointsize = 12, bg = "white")

    r.par(las=1)
    r.plot(lAxeX, lAxeY, yaxs='i', xaxs='i', \
           ylim=lAxisRange, 
           type='l', xlab=sXLabel, ylab=sYLabel,
           main=sMain, xaxt='n', yaxt='n',\
           col='blue',  lty=0, xpd=r.NA)

    draw_grid(min(lAxeX), max(lAxeX), nYMax)

    # Dessinons les lignes
    lMean = []
    for sStation in lStation:
        lYear = dStation[sStation].keys()
        lYear.sort()
        lAxeX = []
        lAxeY = []
        fTotalY = 0
        for sYear in lYear:
            lAxeX.append(int(sYear))
            fYValue = dStation[sStation][sYear]
            fTotalY = fTotalY + fYValue
            lAxeY.append(fYValue)
        # Valeurs 
        r.lines(lAxeX, lAxeY, col=dCouleur[sStation], lwd=2)
        # Valeurs moyennes (-0.5 parce que 2006 est incomplet)
        fMean = round(fTotalY/(float(len(lAxeY))-0.5),2)
        lYMean = [fMean] * len(lAxeY)
        if fMean in lMean:
            nlty = 3
        else:
            nlty = 2
        print sStation, fTotalY, str(fMean).replace('.',',')
#        print 'Total', fTotalY
#        print 'Moyenne', fMean
#        print 'lty', nlty
#        print '----'

        r.lines(lAxeX, lYMean, col=dCouleur[sStation], lty=nlty, lwd=1)
        lMean.append(fMean)
        
    draw_legend(dStation, min(lAxeX), max(lAxeX), nYMax, bSvg)    
    print 'Writing', sPathGraph
    r.dev_off()

def draw_grid(nXMin, nXMax, nYMax):
    """
    Mettre une ligne grise a chaque annee et 
    """
     # Draw the tick
    lXPosTick = range(nXMin, nXMax+1)
    if nYMax < 20:
        lYPosTick = range(0, nYMax)
    else:
        lYPosTick = range(0, nYMax, 5)

    
    r.axis(1, at=lXPosTick)
    r.axis(2, at=lYPosTick)

    # Draw vertical line
    rlUsr = r.par('usr')
    rYdomain =r.c(rlUsr[2], rlUsr[3]) 
    for i in lXPosTick[1:-1]:
        rXdomain = r.c(i, i)
        r.lines(rXdomain, rYdomain, col='gray70')
    # Draw horizontal line
    rXdomain = r.c(rlUsr[0], rlUsr[1])
    for i in lYPosTick[1:]:
        rYdomain = r.c(i,i)
        r.lines(rXdomain, rYdomain, col='gray70')
    # Redraw extremities
    r.lines(r.c(rlUsr[0], rlUsr[0]), \
            rYdomain, col='black')
    r.lines(r.c(rlUsr[1], rlUsr[1]), \
            rYdomain, col='black')


def draw_legend(dStation, nXMin, nXMax, nYMax, bSvg):
    """
    Draw the legend for the graph. 
    """
    if bSvg:
        nCEX=1.2
        dVilles = { 'bagotville': 'Bagotville',
                    'fredericton': 'Fredericton',
                    'halifax' : 'Halifax',
                    'montreal': 'Montr&#233;al',
                    'ottawa': 'Ottawa',
                    'quebec': 'Qu&#233;bec',
                    'toronto' : 'Toronto',
                    'KALB' : 'Albany',
                    'KBTV' : 'Burlington'}        
    else:
        nCEX=0.8
        dVilles = { 'bagotville': 'Bagotville',
                    'fredericton': 'Fredericton',
                    'halifax' : 'Halifax',
                    'montreal': 'Montral',
                    'ottawa': 'Ottawa',
                    'quebec': 'Qubec',
                    'toronto' : 'Toronto',
                    'KALB' : 'Albany',
                    'KBTV' : 'Burlington'}

    nNbrStation = len(dStation.values())
    nHalfValue = nNbrStation/2+1

    # Premiere moitie des stations
    lFullKeys = dStation.keys()
    lFullKeys.sort()
    lFirstHalfKeys = lFullKeys[0:nHalfValue]
    lSecondHalfKeys = lFullKeys[nHalfValue:]

    # Deuxieme moitie des stations
    lCouleur = []
    lTextLegend = []
    for sKey in lFirstHalfKeys:
        lCouleur.append(dCouleur[sKey])
        lTextLegend.append(dVilles[sKey])

    r.legend(nXMin, nYMax, lTextLegend, lwd=2, col=lCouleur,\
             cex=nCEX)

    lCouleur = []
    lTextLegend = []
    for sKey in lSecondHalfKeys:
        lCouleur.append(dCouleur[sKey])
        lTextLegend.append(dVilles[sKey])

    r.legend(nXMin+(nXMax-nXMin)/3, nYMax, lTextLegend, lwd=2, col=lCouleur,\
             cex=nCEX)

    # Encadre pour les lignes pointillees representant la moyenne
    r.legend(nXMax-2.5, nYMax, 'moyenne', lty=2, cex=nCEX)



if __name__ == "__main__":


    if len(sys.argv) < 5:
        print "Usage graph_gel_degel_pluie_interval.py repertoire_station graph.ext nSeuilMin nSeuilMax [pluie]"
        sys.exit(1)


    sDirectory = sys.argv[1]
    sPathGraph = sys.argv[2]
    nSeuilMin = int(sys.argv[3])
    nSeuilMax = int(sys.argv[4])
    if len(sys.argv) > 5 and \
           sys.argv[5] == 'pluie':
        bPluie = True
    else:
        bPluie = False
            
    if 'png' in sPathGraph.split('/')[-1]:
        bSvg = False
    elif 'svg' in sPathGraph.split('/')[-1]:
        bSvg = True
    else:
        print "Extension of last argument must be 'png' or 'svg'!:", sPathGraph
        sys.exit(1)

    main(sDirectory, sPathGraph, bSvg, nSeuilMin, nSeuilMax, bPluie)





