#!/usr/bin/env python
# -*- coding:iso-8859-1  -*-
#
# Code utilise pour l'article http://ptaff.ca/routes/
#
# Copyright (C) 2006 Miguel Tremblay
#
#  Questions or bugs report: miguel.tremblay@ptaff.ca
#
#
# Code contributed by:
#  Miguel Tremblay - ptaff.ca
#
########################################################################
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
#


"""
 Name:	      graph_climato.py
 Description: Prend en entree un fichier texte csv avec comme premiere
  entre l'axe des X en mois et comme deuxieme entre l'axe Y en MM. de pluie.
 
 Notes: 

 Author: Miguel Tremblay
 Date: 15 mai 2006
"""

import sys
from rpy import r
import os.path

lOrdreMois =  range(6,13) + range(1,6)
#lCouleur = ['blue', 'red', 'orange', 'cyan', 'green', 'purple', 'magenta', \
#            'gold', 'purple']
dCouleur = {'bagotville':'blue',
            'ottawa':'red',
            'fredericton':'orange',
            'halifax':'cyan',
            'montreal':'green',
            'quebec':'purple',
            'toronto':'magenta', 
            'KALB':'gold',
            'KBTV':'brown'}
dVilles = { 'bagotville': 'Bagotville',
            'fredericton': 'Fredericton',
            'halifax' : 'Halifax',
            'montreal': 'Montral',
            'ottawa': 'Ottawa',
            'quebec': 'Qubec',
            'toronto' : 'Toronto',
            'KALB' : 'Albany',
            'KBTV' : 'Burlington'}

def main(lFichiersTexte, sPathGraph):
    

    #Ouverture et lecture des fichiers texte
    dFichier = {}
#    dCouleur = {}
    dStation = {}
    for i in range(0, len(lFichiersTexte)):
        sPathFichier = lFichiersTexte[i]
        fichier_texte = open(sPathFichier, 'r')
        lList = fichier_texte.readlines()
        dFichier[sPathFichier] = lList
        dStation[sPathFichier] = sPathFichier.split('/')[-1].split('_')[0]

        
    start_graph(sPathGraph)

    # Dessine un ligne
    for sKey in dFichier.keys():
        lList = dFichier[sKey]
        dPrecip = {}
        for sLine in lList:
            lLine = sLine.split(',')
            nMonth = int(lLine[0])
            fValue = float(lLine[1])
            dPrecip[nMonth] = fValue
        lRightOrder = []
        for nMonth in lOrdreMois:
            lRightOrder.append(dPrecip[nMonth])
        r.lines(range(1,13), lRightOrder, col=dCouleur[dStation[sKey]], lwd=2)

    draw_legend(dStation)

        
    r.dev_off()

def draw_legend(dStation):
    """
    Draw the legend for the graph. dStation contains the string
    of the filename before the first '_' and dCouleur contains the
    color for the data of a specific filenames.
    """
    nNbrStation = len(dStation.values())
    nHalfValue = nNbrStation/2

    lFullKeys = dStation.keys()
    lFullKeys.sort()
    lFirstHalfKeys = lFullKeys[0:nHalfValue]
    lSecondHalfKeys = lFullKeys[nHalfValue:]

    lCouleur = []
    lTextLegend = []
    for sKey in lFirstHalfKeys:
        lCouleur.append(dCouleur[dStation[sKey]])
        lTextLegend.append(dVilles[dStation[sKey]])

    r.legend(1, 199, lTextLegend, lwd=2, col=lCouleur,\
             cex=0.8)

    lCouleur = []
    lTextLegend = []
    for sKey in lSecondHalfKeys:
        lCouleur.append(dCouleur[dStation[sKey]])
        lTextLegend.append(dVilles[dStation[sKey]])

    r.legend(5, 199, lTextLegend, lwd=2, col=lCouleur,\
             cex=0.8)
    
def start_graph(sPngPath):
    """
    Debute le graphique en dissante le cadre.
    """

    # String pour le graphique
    s_x_label = 'Mois'
    s_y_label = 'Prcipitation moyenne (mm)'
    s_main = 'Climatologie du nombre de mm \n de prcipitation liquide par mois'
    
    # L'axe des X ira de 6-12 1-5
    lAxeX =  range(6,13) + range(1,6)
    lAxeY = range(0,200) # cet axe Y est simplement pour avoir un domaine
    r.png(sPngPath, width = 480, height = 480,
         pointsize = 12, bg = "white")
    r.par(las=1)
    r.plot(lAxeX, lAxeX, yaxs='i', xaxs='i', \
           ylim=[min(lAxeY), max(lAxeY)], \
           type='l', xlab=s_x_label, ylab=s_y_label, cex=1.4,
           main=s_main, xaxt='n', yaxt='n',\
           col='blue',  lty=0)
    lXtick = range(1,13)
    lYtick = range(0,200,25)
    r.axis(1, lOrdreMois, at=lXtick)
    r.axis(2, at=lYtick)
    draw_grid()


def draw_grid():
    """
    If dx and/or dy is set, put a gray line on the coorresponding
    axes separated by this value.
    """
    # Draw vertical line
    rlUsr = r.par('usr')
    rYdomain =r.c(rlUsr[2], rlUsr[3]) 
    for i in range(2,12):
        rXdomain = r.c(i, i)
        r.lines(rXdomain, rYdomain, col='gray70')
    # Draw horizontal line
    rXdomain = r.c(rlUsr[0], rlUsr[1])
    for i in range(25,199,25):
        rYdomain = r.c(i,i)
        r.lines(rXdomain, rYdomain, col='gray70')
    # Redraw extremities
    r.lines(r.c(rlUsr[0], rlUsr[0]), \
            rYdomain, col='black')
    r.lines(r.c(rlUsr[1], rlUsr[1]), \
            rYdomain, col='black')


if __name__ == "__main__":


    if len(sys.argv) < 3:
        print "Usage graph_climato.py fichier1.txt fichier2.txt ... graph.png"
        sys.exit(1)


    lFichiersTexte = sys.argv[1:-1]
    sPathGraph = sys.argv[-1]
    if 'png' not in sPathGraph:
        print "Last argument is png path!:", sPathGraph
        sys.exit(1)
    lFichiersTexte.sort()
    main(lFichiersTexte, sPathGraph)




