#!/usr/bin/env python
# -*- coding:iso-8859-1  -*-
#
# Code utilise pour l'article http://ptaff.ca/routes/
#
# Copyright (C) 2006 Miguel Tremblay
#
#  Questions or bugs report: miguel.tremblay@ptaff.ca
#
#
# Code contributed by:
#  Miguel Tremblay - ptaff.ca
#
########################################################################
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#

"""
 Name:	      graph_deg_jour_gel.py
 Description: Prend en entree un fichier texte csv avec comme donnees:
   YYYY, MM, deg_jour_gel

 
 Notes: 

 Author: Miguel Tremblay
 Date: 7 juin 2006
"""

import sys
from rpy import r
import os.path

lOrdreMois =  range(6,13) + range(1,6)
lYear = range(1996,2007)
dCouleur = {'bagotville':'blue',
            'ottawa':'red',
            'fredericton':'orange',
            'halifax':'cyan',
            'montreal':'green',
            'quebec':'purple',
            'toronto':'magenta', 
            'KALB':'gold',
            'KBTV':'brown'}
dVilles = { 'bagotville': 'Bagotville',
            'fredericton': 'Fredericton',
            'halifax' : 'Halifax',
            'montreal': 'Montral',
            'ottawa': 'Ottawa',
            'quebec': 'Qubec',
            'toronto' : 'Toronto',
            'KALB' : 'Albany',
            'KBTV' : 'Burlington'}

sDataStationPath = '/home/dirac/projets/ptaff/routes/data/stations/'

def main(sPathGraph):
    nYMax = 2000

    # Ouverture et lecture des fichiers textes
    dVillesData = {}
    for sStation in dVilles.keys():
        sFichierTexte = sDataStationPath + sStation + '/' + sStation + \
                        '_nbr_degre_jour_gel.txt'
        file_input = open(sFichierTexte, 'r')
        lFichierOriginal = file_input.readlines()
        dVillesData[sStation] = lFichierOriginal


    # Placer les hivers ensembles.
    dHiverData = {}
    for sStation in dVillesData.keys():
        lDataStation = dVillesData[sStation]
        dHiverData[sStation] = {}
        for nYear in lYear:
            fTotalDegreJourGel = 0
            for sLine in lDataStation:
                # Parse the line
                lLine = sLine.split(',')
                # Get year
                nCurrentYear = int(lLine[0])
                # Get month
                nCurrentMonth = int(lLine[1])
                fCurrentDegJourGel = float(lLine[2])
                if nCurrentYear == nYear and nCurrentMonth in range(7,13):
                    fTotalDegreJourGel = fTotalDegreJourGel +\
                                         fCurrentDegJourGel
                elif nCurrentYear == nYear+1 and\
                         nCurrentMonth in range(0,7):
                    fTotalDegreJourGel = fTotalDegreJourGel +\
                                         fCurrentDegJourGel
            dHiverData[sStation][nYear] = fTotalDegreJourGel

    start_graph(sPathGraph, nYMax)
    for sStation in dVillesData.keys():
        X = range(0,len(lYear))
        Y = dHiverData[sStation].values()
        r.lines(X, Y, col=dCouleur[sStation], lwd=2)

    draw_legend(nYMax)
    r.dev_off()
    return


def draw_legend(nYMax):
    """
    Draw the legend for the graph. 
    """
    nNbrStation = len(dVilles.values())
    nHalfValue = nNbrStation/2+1

    lFullKeys = dVilles.keys()
    lFullKeys.sort()
    lFirstHalfKeys = lFullKeys[0:nHalfValue]
    lSecondHalfKeys = lFullKeys[nHalfValue:]

    lCouleur = []
    lTextLegend = []
    for sKey in lFirstHalfKeys:
        lCouleur.append(dCouleur[sKey])
        lTextLegend.append(dVilles[sKey])

    r.legend(0, nYMax-1, lTextLegend, lwd=2, col=lCouleur,\
             cex=0.8)

    lCouleur = []
    lTextLegend = []
    for sKey in lSecondHalfKeys:
        lCouleur.append(dCouleur[sKey])
        lTextLegend.append(dVilles[sKey])

    r.legend(3, nYMax-1, lTextLegend, lwd=2, col=lCouleur,\
             cex=0.8)
    
def start_graph(sPngPath, nYMax):
    """
    Debute le graphique en dessinant le cadre et la grille.
    """

    # String pour le graphique
    s_x_label = 'Hiver'
    sYLabel = "Degrs-jours de gel"
    sMain = "Degrs-jours de gel pour les hivers de 1996  2006"
    # L'axe des X ira de 6-12 1-5
    lAxeX =  range(0, len(lYear)-1)
    lAxeY = range(0,nYMax) # cet axe Y est simplement pour avoir un domaine
    r.png(sPngPath, width = 480, height = 480,
         pointsize = 12, bg = "white")

    r.par(las=1)
    r.plot(lAxeX, lAxeX, yaxs='i', xaxs='i', \
           ylim=[min(lAxeY), max(lAxeY)], \
           type='l', xlab=s_x_label, ylab=sYLabel, \
           main=sMain, xaxt='n', yaxt='n',\
           col='blue',  lty=0)
        
    draw_grid(nYMax)


def draw_grid(nYMax):
    """
    If dx and/or dy is set, put a gray line on the coorresponding
    axes separated by this value.
    """
    # Draw the tick
    lXPosTick = range(0,len(lYear)-1)
    lYPosTick = range(0, nYMax, 250)
    lYTick = lYPosTick

    lXLabel = []
    for nYear in lYear[:-1]:
        sWinterYear = str(nYear) + '\n' + str(nYear+1)
        lXLabel.append(sWinterYear)

    r.axis(1, labels=lXLabel, at=lXPosTick, padj=0.5)
    r.axis(2, labels=lYTick, at=lYPosTick)

    # Draw vertical line
    rlUsr = r.par('usr')
    rYdomain =r.c(rlUsr[2], rlUsr[3]) 
    for i in range(1,len(lYear)-2):
        rXdomain = r.c(i, i)
        r.lines(rXdomain, rYdomain, col='gray70')
    # Draw horizontal line
    rXdomain = r.c(rlUsr[0], rlUsr[1])
    for i in range(250, nYMax,\
                   250):

        rYdomain = r.c(i,i)
        r.lines(rXdomain, rYdomain, col='gray70')
    # Redraw extremities
    r.lines(r.c(rlUsr[0], rlUsr[0]), \
            rYdomain, col='black')
    r.lines(r.c(rlUsr[1], rlUsr[1]), \
            rYdomain, col='black')



if __name__ == "__main__":


    if len(sys.argv) != 2:
        print "Usage graph_deg_jour_gel.py graph.png"
        sys.exit(1)


    sPathGraph = sys.argv[1]
    if 'png' not in sPathGraph:
        print "Last argument is png path!:", sPathGraph
        sys.exit(1)
    main(sPathGraph)




